<?php

    namespace FederationLib\Classes\Configuration;

    class CacheSystemConfiguration
    {
        /**
         * @var bool
         */
        private $client_objects_enabled;

        /**
         * @var int
         */
        private $client_objects_ttl;

        /**
         * @var string
         */
        private $client_objects_server_preference;

        /**
         * @var string
         */
        private $client_objects_server_fallback;

        /**
         * @var bool
         */
        private $peer_objects_enabled;

        /**
         * @var int
         */
        private $peer_objects_ttl;

        /**
         * @var string
         */
        private $peer_objects_server_preference;

        /**
         * @var string
         */
        private $peer_objects_server_fallback;

        /**
         * CacheSystemConfiguration constructor.
         *
         * @param array $configuration
         */
        public function __construct(array $configuration)
        {
            $this->client_objects_enabled = $configuration['cache_system.cache.client_objects_enabled'] ?? false;
            $this->client_objects_ttl = $configuration['cache_system.cache.client_objects_ttl'] ?? 200;
            $this->client_objects_server_preference = $configuration['cache_system.cache.client_objects_server_preference'] ?? 'any';
            $this->client_objects_server_fallback = $configuration['cache_system.cache.client_objects_server_fallback'] ?? 'any';

            $this->peer_objects_enabled = $configuration['cache_system.cache.peer_objects_enabled'] ?? false;
            $this->peer_objects_ttl = $configuration['cache_system.cache.peer_objects_ttl'] ?? 200;
            $this->peer_objects_server_preference = $configuration['cache_system.cache.peer_objects_server_preference'] ?? 'any';
            $this->peer_objects_server_fallback = $configuration['cache_system.cache.peer_objects_server_fallback'] ?? 'any';
        }

        /**
         * Returns True if client cache objects are enabled, false otherwise
         *
         * @return bool
         */
        public function getClientObjectsEnabled(): bool
        {
            return $this->client_objects_enabled;
        }

        /**
         * Returns the TTL for client cache objects
         *
         * @return int
         */
        public function getClientObjectsTtl(): int
        {
            return $this->client_objects_ttl;
        }

        /**
         * Returns the server preference to where client cache objects should be stored
         *
         * @return string
         */
        public function getClientObjectsServerPreference(): string
        {
            return $this->client_objects_server_preference;
        }

        /**
         * Returns the server fallback to where client cache objects should be stored
         *
         * @return string
         */
        public function getClientObjectsServerFallback(): string
        {
            return $this->client_objects_server_fallback;
        }

        /**
         * Returns True if peer cache objects are enabled, false otherwise
         *
         * @return bool
         */
        public function getPeerObjectsEnabled(): bool
        {
            return $this->peer_objects_enabled;
        }

        /**
         * Returns the TTL for peer cache objects
         *
         * @return int
         */
        public function getPeerObjectsTtl(): int
        {
            return $this->peer_objects_ttl;
        }

        /**
         * Returns the server preference to where peer cache objects should be stored
         *
         * @return string
         */
        public function getPeerObjectsServerPreference(): string
        {
            return $this->peer_objects_server_preference;
        }

        /**
         * Returns the server fallback to where peer cache objects should be stored
         *
         * @return string
         */
        public function getPeerObjectsServerFallback(): string
        {
            return $this->peer_objects_server_fallback;
        }
    }