<?php

    namespace FederationLib\Enums\Standard;

    final class EventCode
    {
        /**
         * A generic event code indicates the event is not specific to a particular event type.
         * This could apply to server events such as cleaning up old data or general system
         * messages.
         */
        public const GENERIC = 'GENERIC';

        /**
         * Indicates a client was created.
         */
        public const CLIENT_CREATED = 'CLIENT_CREATED';

        /**
         * Indicates a client was deleted.
         */
        public const CLIENT_DELETED = 'CLIENT_DELETED';

        /**
         * Indicates a client was enabled.
         */
        public const CLIENT_ENABLED = 'CLIENT_ENABLED';

        /**
         * Indicates a client was disabled.
         */
        public const CLIENT_DISABLED = 'CLIENT_DISABLED';

        /**
         * Indicates a client's authentication was enabled.
         */
        public const CLIENT_AUTHENTICATION_ENABLED = 'CLIENT_AUTHENTICATION_ENABLED';

        /**
         * Indicates a client's authentication was disabled.
         */
        public const CLIENT_AUTHENTICATION_DISABLED = 'CLIENT_AUTHENTICATION_DISABLED';

        /**
         * Indicates a client's query document was rejected
         * (e.g. the query document couldn't be processed by the server, see the message for details).
         */
        public const QUERY_DOCUMENT_REJECTED = 'QUERY_DOCUMENT_REJECTED';

        /**
         * Indicates an anomaly was detected in INCOMING events
         * (e.g. a client is receiving too many requests/messages).
         */
        public const ANOMALY_INCOMING = 'ANOMALY_INCOMING';

        /**
         * Indicates an anomaly was detected in OUTGOING events
         * (e.g. a client is sending too many requests/messages).
         */
        public const ANOMALY_OUTGOING = 'ANOMALY_OUTGOING';

        /**
         * Indicates an anomaly was detected in PEER JOIN events
         * (e.g. a channel is receiving too many join requests/events).
         */
        public const ANOMALY_PEER_JOIN = 'ANOMALY_PEER_JOIN';

        /**
         * Indicates a service cleanup event.
         * This is a special event code that is not intended to be used by clients.
         * (e.g. the server is cleaning up old data, results are explained in the message).
         */
        public const SERVICE_CLEANUP = 'SERVICE_CLEANUP';

        /**
         * Indicates a service message event.
         * This is a special event code that is not intended to be used by clients.
         * (e.g. a message from the server).
         */
        public const SERVICE_MESSAGE = 'SERVICE_MESSAGE';

        /**
         * Indicates a service error event.
         * This is a special event code that is not intended to be used by clients.
         * (e.g. an error occurred on the server, results are explained in the message).
         */
        public const SERVICE_ERROR = 'SERVICE_ERROR';

        /**
         * Indicates a peer was restricted by the server.
         * This is a special event code that is not intended to be used by clients.
         * (e.g. a peer was restricted by the server, results are explained in the message).
         */
        public const PEER_RESTRICTED = 'PEER_RESTRICTED';

        /**
         * An array of all event codes.
         */
        public const ALL = [
            self::GENERIC,
            self::CLIENT_CREATED,
            self::CLIENT_DELETED,
            self::CLIENT_ENABLED,
            self::CLIENT_DISABLED,
            self::CLIENT_AUTHENTICATION_ENABLED,
            self::CLIENT_AUTHENTICATION_DISABLED,
            self::QUERY_DOCUMENT_REJECTED,
            self::ANOMALY_INCOMING,
            self::ANOMALY_OUTGOING,
            self::ANOMALY_PEER_JOIN,
            self::SERVICE_CLEANUP,
            self::SERVICE_MESSAGE,
            self::SERVICE_ERROR,
            self::PEER_RESTRICTED
        ];
    }